// (c) Thomas Kamps, 2009
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <QLabel>
#include <QPainter>
#include <QStackedLayout>
#include <QMouseEvent>
#include "field.h"
#include "src/Monopd/monopdfield.h"
#include "ui_field.h"
#include "cmdtoolbutton.h"
#include "auction.h"

Field::Field(QWidget* parent) : QWidget(parent), ui(new Ui::Field) {
	ui->setupUi(this);
	text = new QTextBrowser();
	auction = new Auction();
	stack = new QStackedLayout();
	stack->addWidget(text);
	stack->addWidget(auction);
	ui->center->addLayout(stack);
	stack->setCurrentIndex(0);

	ui->wEstate->setVisible(false);
	ui->fButtons->setVisible(false);
	selectedField = 0;
}

Field::~Field() {
	delete ui;
}

void Field::setGame(MonopdGame* g) {
	game = g;
}

QRect Field::getAvatarRect(int pos) {
	int N = game->getNumFields();
	QRect fRect = getFieldRect(pos);
	int side = 4*pos/N;

	int w;
	int h;

	if (side%2 == 0) {
		w = fRect.width()/2;
		h = fRect.height()/4;
	} else {
		w = fRect.width()/4;
		h = fRect.height()/2;
	}
	int s = w;
	if (h < w) s = h;

	if (side == 0) {
		//Souht side
		return QRect(fRect.x()+fRect.width()/4, fRect.y()+3*fRect.height()/8, s, s);
	}
	if (side == 1) {
		//West side
		return QRect(fRect.x()+3*fRect.width()/8, fRect.y()+fRect.height()/4, s, s);
	}
	if (side == 2) {
		//North side
		return QRect(fRect.x()+fRect.width()/4, fRect.y()+3*fRect.height()/8, s, s);
	}
	if (side == 3) {
		//East side
		return QRect(fRect.x()+3*fRect.width()/8, fRect.y()+fRect.height()/4, s, s);
	}
	return QRect(0,0,0,0);
}

//calculates the rect for teh Field wiht id fid
QRect Field::getFieldRect(int fid) {
	int N = game->getNumFields();

	int w = this->width()-1;
	int h = this->height()-1;
	int s = w;
	if (h < w) s = h;

	//Get sizes of the fields
	int rasterSize = N/4+3;
	int horFieldWidth = s/rasterSize;
	int vertFieldHeight = s/rasterSize;
	int horFieldHeight = vertFieldHeight*2;
	int vertFieldWidth = horFieldWidth*2;

	int xOffset = (w-horFieldWidth*rasterSize)/2;
	int yOffset = (h-vertFieldHeight*rasterSize)/2;

	int side = 4*fid/N;

	int fWidth;
	int fHeight;
	int fposX;
	int fposY;

	//get
	if (fid%(N/4) == 0) {
		//Edge
		fWidth = vertFieldWidth;
		fHeight = horFieldHeight;
	} else {
		if (side%2 == 0) {
			//We are on the North or south side
			fWidth = horFieldWidth;
			fHeight = horFieldHeight;
		} else {
			fWidth = vertFieldWidth;
			fHeight = vertFieldHeight;
		}
	}

	if (side == 0) {
		//South side
		fposX = vertFieldWidth+(N/4-1-fid)*horFieldWidth;
		fposY = (N/4+1)*vertFieldHeight;
	}
	if (side == 1) {
		//West side
		fposX = 0;
		fposY = horFieldHeight+(N/2-1-fid)*vertFieldHeight;
	}
	if (side == 2) {
		//North side
		if (fid > N/2) fposX = vertFieldWidth+(fid-N/2-1)*horFieldWidth;
		else fposX = 0;
		fposY = 0;
	}
	if (side == 3) {
		//East side
		fposX = (N/4+1)*horFieldWidth;
		if (fid > 3*N/4) fposY = horFieldHeight+(fid-3*N/4-1)*vertFieldHeight;
		else fposY = 0;
	}
	return QRect(fposX+xOffset, fposY+yOffset, fWidth, fHeight);
}

void Field::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
		case QEvent::LanguageChange:
			ui->retranslateUi(this);
			break;
		default:
			break;
	}
}

void Field::mousePressEvent(QMouseEvent* event) {
	if (event->button() == Qt::LeftButton) {
		//Get the Field that got clicked
		selectedField = 0;
		for (int i = 0; i < game->getNumFields(); i++) {
			MonopdField* f = game->getField(i);
			if (f->isInRect(event->x(), event->y())) selectedField = f;
		}
		if (selectedField == 0) emit(fieldSelected(0));
		else {
			if (selectedField->getGroup() == -1) {
				selectedField = 0;
				emit(fieldSelected(0));
			} else {
				emit(fieldSelected(selectedField));
			}
		}
		update();
	}
	//TODO: React on right-click
}

void Field::emitButtonClick(QString cmd) {
	emit(buttonClicked(cmd));
	clearButtons();
}

void Field::showField(int fid) {
	if (fid == -1) {
		//clear Estate
		ui->wEstate->setVisible(false);
	} else {
		MonopdField* f = game->getField(fid);
		int r = f->getColor().red();
		int g = f->getColor().green();
		int b = f->getColor().blue();
		QString sr = QString::number(r, 16);
		QString sg = QString::number(g, 16);
		QString sb = QString::number(b, 16);
		if (sr.size() < 2) sr = "0"+sr;
		if (sg.size() < 2) sg = "0"+sg;
		if (sb.size() < 2) sb = "0"+sb;
		QString col = "#"+sr+sg+sb;
		if (f->getGroup() != -1) {
			if (f->getColor().alpha() == 0) {
				ui->lEstate->setText("<span style=\"font-size:20pt; font-weight:bold\">"+f->getName()+"</span>");
			} else {
				ui->lEstate->setText("<span style=\"background:"+col+"; font-size:20pt; font-weight:bold\">"+f->getName()+"</span>");
			}

		ui->lPrice->setText("<span style=\"font-size:20pt; font-weight:bold\">"+QString::number(f->getPrice())+"</span>");
		} else {
			ui->lEstate->setText("<span style=\"font-size:20pt; font-weight:bold\">"+f->getName()+"</span>");
			ui->lPrice->setText("");
		}
		ui->wEstate->setVisible(true);
	}
}

void Field::appendLine(QString txt) {
	text->append(txt);
}

void Field::addButton(QString text, QString cmd) {
	CmdToolButton* b = new CmdToolButton();
	b->setText(text);
	b->cmd = cmd;
	if (cmd == ".eb") b->setIcon(QIcon(":/images/field/document.png"));
	if (cmd == ".ea") b->setIcon(QIcon(":/images/tools/auction.png"));
	if (cmd == ".E") b->setIcon(QIcon(":/images/tools/endturn.png"));

	if (cmd == ".jr") b->setIcon(QIcon(":/images/tools/jailroll.png"));
	if (cmd == ".jc") b->setIcon(QIcon(":/images/tools/jailcard.png"));
	if (cmd == ".jp") b->setIcon(QIcon(":/images/tools/jailpay.png"));

	b->setIconSize(QSize(24,24));
	connect(b, SIGNAL(cmdClicked(QString)), this, SLOT(emitButtonClick(QString)));
	buttons.append(b);
	ui->buttons->insertWidget(buttons.size(), b);
	ui->fButtons->setVisible(true);
}

void Field::clearText() {
	text->setText("");
	clearButtons();
}

void Field::clearButtons() {
	while (buttons.size() > 0) {
		CmdToolButton* b = buttons.value(0);
		buttons.removeAll(b);
		delete b;
	}
	ui->fButtons->setVisible(false);
}

void Field::addPlayerAuction(Player* p) {
	auction->addPlayerWidget(p->auction);
}

void Field::showAuction() {
	stack->setCurrentIndex(1);
	for (int i = 0; i < game->getNumPlayer(); i++) {
		Player* pl = game->getPlayerFromList(i);
		pl->auction->setName(pl->getName());
		pl->auction->setMoney("0");
	}
	auction->resetSpinBox();
}

void Field::updatAuctionId(int aid) {
	auction->setId(aid);
}

void Field::updateAuctionField(int fid) {
	auction->setFieldName(game->getField(fid)->getName());
}

void Field::bidUpdate(int pid, int bid) {
	Player* p = game->getPlayer(pid);
	if (p == 0) return;
	for (int i = 0; i < game->getNumPlayer(); i++) {
		Player* pl = game->getPlayerFromList(i);
		pl->auction->setName(pl->getName());
	}
	p->auction->setName("<b>"+p->getName()+"</b>");
	p->auction->setMoney(QString::number(bid));
	auction->setMinBid(bid+1);
}

void Field::updateAuctionStatus(int status) {
	switch (status) {
		case 0:
			auction->setStatusText("");
			break;
		case 1:
			auction->setStatusText("First...");
			break;
		case 2:
			auction->setStatusText("Second...");
			break;
		case 3:
			stack->setCurrentIndex(0);
			break;
		default:
			break;
	}
}

void Field::paintEvent(QPaintEvent* pe) {
	int N = game->getNumFields();
	int w = this->width()-1;
	int h = this->height()-1;
	if (w < h) h = w;
	if (h < w) w = h;

	QPainter p(this);

	//Set size of widget in Field-center
	int rasterSize = N/4+3;
	int horFieldWidth = w/rasterSize;
	int vertFieldHeight = h/rasterSize;

	//Now draw South-Fields, we draw from left to right
	for (int i = 0; i < N; i++) {
		int side = 4*i/N;

		//Get Data for drawing
		MonopdField* f = game->getField(i);
		QRect fRect = getFieldRect(i);
		f->setFieldRect(fRect);

		int fposX = fRect.x();
		int fposY = fRect.y();
		int fWidth = fRect.width();
		int fHeight = fRect.height();

		QRect strRect; //Drawing rect for the street-color
		QRect fName; //Rect for the Fieldname

		QRect docRect; //Rect for document or owner-avatar
		int houseStartX; //  Position of the houses
		int houseStartY; //
		int houseWidth;  //   Sizes of a house
		int houseHeight; //
		int houseOffsetX; //  Offset for house drawing
		int houseOffsetY; //
		if (side == 0) {
			//South side
			strRect = QRect(fposX, fposY, fWidth, fHeight/4);
			fName = QRect(fposX, fposY+fHeight/4, fWidth, 5*fHeight/8);
			docRect = QRect(fposX+3*fWidth/4, fposY+7*fHeight/8, fWidth/4, fHeight/8);
			houseStartX = fposX,
			houseStartY = fposY+2;
			houseWidth = fWidth/4;
			houseHeight = fHeight/4-4;
			houseOffsetX = fWidth/4;
			houseOffsetY = 0;
		} else if (side == 1) {
			//West side
			strRect = QRect(fposX+3*fWidth/4, fposY, fWidth/4, fHeight);
			fName = QRect(fposX, fposY, 3*fWidth/4, 3*fHeight/4);
			docRect = QRect(fposX, fposY+3*fHeight/4, fWidth/8, fHeight/4);
			houseStartX = fposX+3*fWidth/4+2,
			houseStartY = fposY;
			houseWidth = fWidth/4-4;
			houseHeight = fHeight/4;
			houseOffsetX = 0;
			houseOffsetY = fHeight/4;
		} else if (side == 2) {
			//North side
			strRect = QRect(fposX, fposY+3*fHeight/4, fWidth, fHeight/4);
			fName = QRect(fposX, fposY+fHeight/8, fWidth, 5*fHeight/8);
			docRect = QRect(fposX, fposY, fWidth/4, fHeight/8);
			houseStartX = fposX,
			houseStartY = fposY+3*fHeight/4+2;
			houseWidth = fWidth/4;
			houseHeight = fHeight/4-4;
			houseOffsetX = fWidth/4;
			houseOffsetY = 0;
		} else if (side == 3) {
			//East side
			strRect = QRect(fposX, fposY, fWidth/4, fHeight);
			fName = QRect(fposX+fWidth/4, fposY, 5*fWidth/8, fHeight);
			docRect = QRect(fposX+7*fWidth/8, fposY, fWidth/8, fHeight/4);
			houseStartX = fposX+2,
			houseStartY = fposY;
			houseWidth = fWidth/4-4;
			houseHeight = fHeight/4;
			houseOffsetX = 0;
			houseOffsetY = fHeight/4;
		}

		if (f != 0) {
			if (f->getOwner() != -1) {
				//Draw light green, if field is owned
				p.setBrush(QColor(192,255,192));
			} else {
				//Otherwise make white
				p.setBrush(QColor(255,255,255));
			}
			p.drawRect(fposX, fposY, fWidth, fHeight);

			//Draw field name
			p.setPen(Qt::black);
			p.drawText(fName, Qt::AlignCenter|Qt::TextWrapAnywhere, f->getName());

			if (f->getGroup() != -1) {
				//Draw streetColor
				p.setPen(f->getColor());
				p.setBrush(f->getColor());
				p.drawRect(strRect);

				//draw Houses or Hotel
				p.setPen(Qt::black);
				if (f->getHouses() < 5) {
					p.setBrush(Qt::green);
					for (int k = 0; k < f->getHouses(); k++) {
						p.drawRect(houseStartX+k*houseOffsetX, houseStartY+k*houseOffsetY, houseWidth, houseHeight);
					}
				} else {
					p.setBrush(Qt::red);
					p.drawRect(houseStartX, houseStartY, houseWidth+houseOffsetX, houseHeight+houseOffsetY);
				}
			}

			if (f->getBuyable()) {
				//Draw document or (TODO) avatar of owner
				QPixmap pix;
				if (f->getOwner() == -1) {
					pix = QPixmap(":/images/field/document.png");
				} else {
					pix = QPixmap(":/images/avatars/"+game->getPlayer(f->getOwner())->getAvatar());
				}
				p.drawPixmap(docRect, pix);
				if (f->getMortaged()) {
					p.setBrush(QColor(0,0,0,128));
					p.drawRect(fposX, fposY, fWidth, fHeight);
				}
			}
		}

		//Draw Field Borders
		p.setPen(QColor(0,0,0));
		p.setBrush(Qt::NoBrush);
		p.drawRect(fposX, fposY, fWidth, fHeight);
	}

	//Draw selectedField
	if (selectedField != 0) {
		p.setBrush(QColor(128,128,255,64));
		p.drawRect(selectedField->getFieldRect());
	}

	//Draw Players
	for (int i = 0; i < game->getNumPlayer(); i++) {
		Player* pl = game->getPlayerFromList(i);
		if (pl == 0) continue;
		if (pl->getGame() == game->getGameId()) {
			QRect avRect = getAvatarRect(pl->getPos());
			QPixmap pix = QPixmap(":/images/avatars/"+pl->getAvatar());
			p.drawPixmap(avRect, pix);
			if (pl->getInJail()) p.drawPixmap(avRect, QPixmap(":/images/field/jailed.png"));
		}
	}


	p.end();

	ui->centWidget->setMaximumWidth((N/4-2)*horFieldWidth);
	ui->centWidget->setMaximumHeight((N/4-2)*vertFieldHeight);
	ui->centWidget->setMinimumWidth((N/4-2)*horFieldWidth);
	ui->centWidget->setMinimumHeight((N/4-2)*vertFieldHeight);
}
